"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = __importDefault(require("assert"));
require("./codeMirrorEnv");
var automerge_1 = __importDefault(require("automerge"));
var connectAutomergeDoc_1 = __importDefault(require("../src/connectAutomergeDoc"));
var codemirror_1 = __importDefault(require("codemirror"));
describe('automergeCodeMirror', function () {
    var watchableDoc;
    var getText = function (doc) { return doc.text; };
    var host;
    var codeMirror;
    beforeEach(function () {
        watchableDoc = new automerge_1.default.WatchableDoc(automerge_1.default.from({ text: new automerge_1.default.Text() }));
        host = document.body.appendChild(document.createElement('div'));
        codeMirror = codemirror_1.default(host);
    });
    afterEach(function () {
        host.remove();
    });
    describe('Automerge => CodeMirror', function () {
        it('handles 2 consecutive Automerge changes', function () {
            var connectCodeMirror = connectAutomergeDoc_1.default(watchableDoc);
            var disconnectCodeMirror = connectCodeMirror(codeMirror, getText);
            watchableDoc.set(automerge_1.default.change(watchableDoc.get(), function (draft) { return draft.text.insertAt(0, 'hello'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            watchableDoc.set(automerge_1.default.change(watchableDoc.get(), function (draft) { return draft.text.insertAt(0, 'world'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'worldhello');
            disconnectCodeMirror();
        });
        it('ignores Automerge changes after disconnection', function () {
            var connectCodeMirror = connectAutomergeDoc_1.default(watchableDoc);
            var disconnectCodeMirror = connectCodeMirror(codeMirror, getText);
            watchableDoc.set(automerge_1.default.change(watchableDoc.get(), function (draft) { return draft.text.insertAt(0, 'hello'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            disconnectCodeMirror();
            watchableDoc.set(automerge_1.default.change(watchableDoc.get(), function (draft) { return draft.text.insertAt(0, 'world'); }));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
        });
    });
    describe('CodeMirror => Automerge', function () {
        it('handles 2 consecutive CodeMirror changes', function () {
            var connectCodeMirror = connectAutomergeDoc_1.default(watchableDoc);
            connectCodeMirror(codeMirror, getText);
            codeMirror.replaceRange('hello', codeMirror.posFromIndex(0));
            assert_1.default.strictEqual(codeMirror.getValue(), 'hello');
            assert_1.default.strictEqual(watchableDoc.get().text.toString(), 'hello');
            codeMirror.replaceRange('world', codeMirror.posFromIndex(0));
            assert_1.default.strictEqual(codeMirror.getValue(), 'worldhello');
            assert_1.default.strictEqual(watchableDoc.get().text.toString(), 'worldhello');
        });
    });
});
//# sourceMappingURL=automergeCodeMirrorTest.js.map